/*
**
**  bestpath.c - Empire autorouting functions
**
**  Initial version 1989 by Bill Frolik & Jay Phillips
**  Revised 1990, 1991 by Bill Frolik
**
*/

#include <stdio.h>
#include "gamesdef.h"
#include "misc.h"
#include "bigmap.h"
#include "sect.h"

/*
**  Global variables used in this module
*/

struct {
    char pathsectors[SCT_MAXDEF+1];	/* legal enroute sector types */
    char destsectors[SCT_MAXDEF+1];	/* legal target sector types */
    int destx,desty;			/* -or- target sector coords */
} autoroute;

char finalpath[WORLDSIZE/2];
char *dir = "gyujnbh";
char ix[6] = {-2,-1, 1, 2, 1,-1};
char iy[6] = { 0,-1,-1, 0, 1, 1};
int isdest();
int ispath();
unsigned short nextindex[WORLDSIZE];
char dirtohere[WORLDSIZE];

/*
**  External variables used by this module
*/

extern char bigmap[];

/* ________________________________________________________________
**|                                                                |
**| TRACE(p0,p1,p2,p3,p4,p5,p6,p7,p8,p9)                           |
**|                                                                |
**| Used just like printf, but creates/writes to "trace.emp".      |
**|________________________________________________________________|
*/

#ifdef FOOBAR
TRACE(p0,p1,p2,p3,p4,p5,p6,p7,p8,p9)
char *p0,*p1,*p2,*p3,*p4,*p5,*p6,*p7,*p8,*p9;
{
    static FILE *tfp = 0;
    if (!tfp) tfp = fopen("trace.emp","w");
    if (tfp) {
	fprintf(tfp,p0,p1,p2,p3,p4,p5,p6,p7,p8,p9);
	fflush(tfp);
    }
}
#else
#define TRACE (void)
#endif


/* ________________________________________________________________
**|                                                                |
**| legalroute(p,d,dx,dy) - set up the autoroute structure         |
**|								   |
**| Specify:							   |
**|	p     = string of valid path sector types		   |
**|	d     = string of valid destination sector types           |
**|     dx,dy = destination coordinates (if d = null string)       |
**|								   |
**| Returns TRUE if the parameters make sense and are valid.	   |
**|________________________________________________________________|
*/

legalroute(path,dest,dstx,dsty)
char *path,*dest;
int dstx,dsty;
{
    if (strlen(path)>SCT_MAXDEF || strlen(dest)>SCT_MAXDEF) return(0);
    strcpy(autoroute.pathsectors,path);
    strcpy(autoroute.destsectors,dest);
    autoroute.destx = xnorm(dstx);
    autoroute.desty = ynorm(dsty);
    TRACE("legalroute: setting valid path sectors to '%s'\n",
	  autoroute.pathsectors);
    TRACE("            destination is either '%s' or (%d,%d)\n",
	  autoroute.destsectors, 
	  autoroute.destx,autoroute.desty);
    if (!*dest) return(valid(autoroute.destx,autoroute.desty));
    return(1);
}

/* ________________________________________________________________
**|                                                                |
**| legalpath(x,y) - return TRUE if (x,y) is a valid path sector   |
**|________________________________________________________________|
*/

validterrain(c,s)
char c,*s;
{
    int negate;
    if (!s || !*s) return(1);
    if (negate = (*s=='~')) s++;
    while (*s && c!=*s) s++;
    return(negate ? !*s : *s);
}

legalpath(x,y)
int x,y;
{
    return(validterrain(stype(x,y),autoroute.pathsectors));
}

/* ________________________________________________________________
**|                                                                |
**| legaldest(x,y) - return TRUE if (x,y) is a valid dest sector   |
**|________________________________________________________________|
*/

legaldest(x,y)
int x,y;
{
    char c,*s;
    int negate;
    s = autoroute.destsectors;
    if (*s) {
	c = stype(x,y);
	if (negate = (*s=='~')) s++;
	while (*s && c!=*s) s++;
	return(negate ? !*s : *s);
    }
    return((x==autoroute.destx) && (y==autoroute.desty));
}

/* ___________________________________________________________________
**|                                                                   |
**| (char *)shortestpath(sx,sy,isdest(),ispath(),maxpathlen)          |
**|                                                                   |
**| Find the most direct path, if one exists, from sector (sx,sy) to  |
**| whatever sector meets certain destination criteria, following a   |
**| route that meets certain path criteria.                           |
**|								      |
**| Specify - 							      |
**| 								      |
**|     sx,sy		Starting sector coordinates                   |
**|     isdest()        Valid destination sector check routine        |
**|     ispath()        Valid path sector check routine               |
**|     maxpathlen      Maximum allowable path length (0=infinite)    |
**|								      |
**| Returns -							      |
**|								      |
**|	null string	if path not possible		              |
**|     null pointer    if (sx,sy) coordinates are invalid            |
**|	"?"		if path excees maximum allowable length       |
**|     "gyujnbh"       routing string if path is possible            |
**|							              |
**| If a valid route is found, the returned routing string will       |
**| contain the usual combination of [g,y,u,j,n,b] movement codes,    |
**| and will always be terminated by an "h".  If the starting point   |
**| qualifies as a possible destination, the routing string returned  |
**| will simply be a lone "h".                                        |
**|___________________________________________________________________|
*/

char * shortestpath(x,y,isdest,ispath,maxpathlen)
int x,y,maxpathlen;
int (*isdest)(),(*ispath)();
{
    int toofar,atdest;
    int testx,testy,testi;
    int gonearound,perimeterlen,pathlen;
    int i,j,lasti,start;
    char *s;

    /*
    **  Normalize and validate starting coordinates.
    **  Also check if source == destination.
    */

    x = xnorm(x);
    y = ynorm(y);
    i = mindex(x,y);

    TRACE("shortestpath: starting at cell %d(%d,%d)\n",i,x,y);

    if (!valid(x,y))
	return((char *)0);

    if (isdest(x,y)) {
	strcpy(finalpath,"h");
	return(finalpath);
    }
    
    TRACE("%d(%d,%d) is valid and is not destination\n",i,x,y);

    /*
    **  Clear workspace buffers.
    */

    for (j = 0; j < WORLDSIZE; j++) {
	nextindex[j] = 0;
	dirtohere[j] = -1;
    }

    /*
    **  Diffuse outward from the source into all possible
    **  path sectors.  Each cycle through the "for (i=start"
    **  look below checks and grows the perimeter "ring" (and 
    **  hence the final path length) by one sector.  The "ring"
    **  sectors are maintained as a linked list.  We are done 
    **  when we either [1] expand beyond the maximum allowable 
    **  path, [2] encounter a sector that qualifies as the target,
    **  or [3] become trapped and cannot expand any further.
    */

    nextindex[i] = start = i;
    perimeterlen = 1;
    dirtohere[i] = 6;
    atdest = pathlen = toofar = 0;
    TRACE("map matrix ASIZExBSIZE=%dx%d\n",ASIZE,BSIZE);
    while (!atdest && !toofar && perimeterlen) {
	pathlen++;
	if (maxpathlen && (pathlen > maxpathlen)) toofar++;
	else {
	    TRACE("checking at pathlen=%d, start=%d\n",pathlen,start);
	    perimeterlen = gonearound = 0;
	    for (i=start; !gonearound && !atdest; i=nextindex[i]) {
/*
		y = i % BSIZE;
		x = (i / BSIZE) * 2 + (y & 1); 
*/
		y = i / ASIZE;
		x = (i % ASIZE) * 2 + (y & 1);

		TRACE("  scanning around [%c] at %d(%d,%d) (next cell=%d)\n",
		      stype(x,y),i,x,y,nextindex[i] );
		for (j=0;j<6 && !atdest;j++) {
		    testx = xnorm(x+ix[j]);
		    testy = ynorm(y+iy[j]);
		    testi = mindex(testx,testy);
		    TRACE("    '%c' leads to [%c] at %d(%d,%d), ",
			  dir[j],stype(testx,testy),testi,testx,testy);
		    if (dirtohere[testi] >= 0) {
			TRACE("already marked unusable\n");
		    }
		    else if (isdest(testx,testy)) {
			TRACE("*BINGO*\n");
			dirtohere[testi] = j;
			atdest = 1;
		    }
		    else if (ispath(testx,testy)) {
			TRACE("ok\n");
			dirtohere[testi] = j;
			if (!perimeterlen) start = testi;
			else nextindex[lasti] = testi;
			lasti = testi;
			perimeterlen++;
		    }
		    else {
			TRACE("newly marking as unusable\n");
			dirtohere[testi] = 6;
		    }
		}
		gonearound = (nextindex[i] == i);
	    }
    	    if (!atdest && !toofar && perimeterlen)
	        nextindex[lasti] = lasti;
	}
    }

    /*
    **  If path is not possible, or might be possible but exceeds
    **  the maximum path length limit, return the appropriate string.
    */

    if (toofar || !atdest) {
	TRACE(toofar ? "maxpathlen exceeded\n" : "no route possible\n");
	strcpy(finalpath,toofar ? "?" : "");
	return(finalpath);
    }

    /*
    **  We have a final path, but it needs to be "decoded".
    **  Retrace the final path from destination back to source,
    **  constructing the final path string as we go.
    */

    TRACE("backtracking:\n");
    s = finalpath+pathlen;
    for (*s='\0'; s>finalpath;) {
	i = dirtohere[testi];
	*--s = dir[i];
	TRACE("  cell %d (%d,%d)\n",testi,testx,testy);
	testx = xnorm(testx-ix[i]);
	testy = ynorm(testy-iy[i]);
	testi = mindex(testx,testy);
    }

    /*
    **  Insure we don't pass back a string full of 'h's.
    **  (Shouldn't happen, but doesn't hurt to play it safe.)
    */

    for (s=finalpath; *s && *s!='h'; s++);
    if (*s == 'h') s[1]='\0';

    TRACE("final path: \"%s\"\n",finalpath);
    return(finalpath);
}

/* _________________________________________________________________________
**|                                                                         |
**| bestpath(x1,y1,x2,y2,(char *)terrain);				    |
**|                                                                         |
**| Calculate routing string to get from sector [x1,y1] to sector [x2,y2]   |
**| via a specified type of terrain.                                        |
**|                                                                         |
**| Specify:                                                                |
**|                                                                         |
**|    x1,y1	    starting coordinates                                    |
**|                                                                         |
**|    x2,y2	    destination coordinates                                 |
**|                                                                         |
**|    terrain	    ptr to string showing the types of sectors that         |
**|                 we're allowed to pass through:                          |
**|                                                                         |
**|                 A null string enables routing through any kind of       |
**|                 sector (useful for airplanes).                          |
**|                                                                         |
**|                 A string that begins with an 'R' ensures that           |
**|                 the destination sector also matches the specified       |
**|                 type of terrain.                                        |
**|                                                                         |
**|                 A string that begins with a '~' (after the 'R',         |
**|                 if necessary) specifies that we can pass through        |
**|                 any kind of sector EXCEPT those in the remainder        |
**|                 of the string.                                          |
**|                                                                         |
**|                 Examples:                                               |
**|                                                                         |
**|                    "R~.^"	all sectors along route must be             |
**|                             non-ocean, non-mountain                     |
**|                                                                         |
**|                    "+"	all sectors between start and end           |
**|                             must be highway                             |
**|                                                                         |
**|                    "h. "	all sectors along route must be             |
**|                             harbor, water, or unmapped                  |
**|                                                                         |
**| 'bestpath' returns a pointer to a route string containing either:       |
**|                                                                         |
**|    yugjbn - string of normal routing characters if route possible       |
**|    ?      - if route is longer than MAXROUTE characters                 |
**|    \0     - (null string) if no route possible                          |
**|    h      - if start and end points are the same sector                 |
**|_________________________________________________________________________|
*/

char * bestpath(sx,sy,dx,dy,terrain)
int sx,sy,dx,dy;
char *terrain;
{
    int restrict;
    if (restrict = (*terrain == 'R')) terrain++;	
    sx = xnorm(sx);
    sy = ynorm(sy);
    if (!valid(sx,sy)) return((char *)0);

    /*
    **  Set up the autoroute structure for shortestpath():
    **  Path must adhere to the specified terrain;
    **  Dest has specific coordinates.
    */

    if (!legalroute(terrain,"",dx,dy)) return((char *)0);

    /*
    **  If 'R' specified, check that the destination
    **  is a valid path sector as well.  Then go ahead
    **  and calculate the final path.
    */
       
    if (restrict) if (!legalpath(dx,dy)) return((char *)0);
    return((char *)shortestpath(sx,sy,legaldest,legalpath,MAXROUTE));
}

