#ifndef lint
static char *RCSid = "$Header: fire.c,v 1.1 89/12/14 08:19:49 jay Exp $";
#endif /* not lint */

/*
 * fire.c
 *
 * fire at other sectors, ships.
 *
 * from PSL Empire, 1985
 */

#include "misc.h"
#include "var.h"
#include "xy.h"
#include "treaty.h"
#include "nat.h"
#include "ship.h"
#include "sect.h"
#include "news.h"
#include "nsc.h"
#include "file.h"
#include <ctype.h>

enum targ_type { land_based, ship_based };

static int fire_issector();

fire()
{
    extern char *argp[];
    char vbuf[10],fbuf[10],buf[80];
    char *ptr;
    double range,prb;
    coord fx,fy,x,y;
    int	btused,mil,gun,fort,shell,shots,milneeded;
    int	trange,guneff,dam,def,vdef,tsub,flev,vlev;
    int	fshipno,vshipno;
    natid vict;
    struct shpstr fship,vship;
    struct sctstr fsect,vsect;
    extern int fire_mob_cost;
    extern double mountain_range;
    extern double nonfort_range;
    enum targ_type target, attacker;
    char linebuf[80];
    
    tsub = 0;

    /* 
    **  Qualify and get info about the victim.
    */

    ptr = getstarg(argp[1], "Firing at? ");
    if (!ptr || !*ptr) return RET_SYN;
    (void)strcpy(vbuf, ptr);
    if (fire_issector(vbuf)) {

	/* TARGET ON LAND */

	if (!sarg_xy(vbuf, &x, &y) || !getsect(x, y, &vsect)) {
	    pr("No such sector exists!\n");
	    return RET_SYN;
	}
	if (vsect.sct_type == SCT_SANCT) {
	    pr(fmt("%s is a %s!!\n",vbuf,dchr[SCT_SANCT].d_name));
	    return RET_FAIL;
	}
	if (vsect.sct_type == SCT_WATER) {
	    pr(fmt("You must specify a ship in sector %s!\n",vbuf));
	    return RET_FAIL;
	}
	tsub = 0;
	target = land_based;
	vdef = landdef((int)vsect.sct_type);
	vict = vsect.sct_own;
#ifdef ELEVATIONS
	vlev = max(0,vsect.sct_elev);
#else
	vlev = (vsect.sct_type==SCT_MOUNT) ? 1 : 0;
#endif
	ptr = fmt("Firing on sector %s from? ",xyas(x,y,cnum));

    } else {

	/* TARGET AT SEA */

	vshipno = atoi(vbuf);
	if (vshipno<0 || 
	    !getship(vshipno,&vship) || 
	    !vship.shp_own ||
	    vship.shp_aboard >= 0) {
	    pr("Not a possible target!\n");
	    return RET_FAIL;
	}
	tsub = issub(&vship);
	target = ship_based;
	vdef = seadef(vship.shp_type);
	vict = vship.shp_own;
	x = vship.shp_x;
	y = vship.shp_y;
#ifdef ELEVATIONS
	if (getsect(x,y,&vsect)) vlev = max(0,vsect.sct_elev); 
	else vlev = 0;
#else
	vlev = 0;
#endif
	ptr = fmt("Fire on %s #%d from? ",
		  class[mchr[vship.shp_type].m_class].name,vshipno);
    }

    /* 
    **  Qualify and get info about the attacker.
    */

    ptr = getstarg(argp[2],ptr);
    if (!ptr || !*ptr) return RET_SYN;
    (void)strcpy(fbuf, ptr);
    if (fire_issector(fbuf)) {

	/* FIRE FROM LAND */

	if (!sarg_xy(fbuf, &fx, &fy) || !getsect(fx, fy, &fsect))
	    return RET_SYN;
	if (fsect.sct_own != cnum) {
	    pr("Not your sector.\n");
	    return RET_FAIL;
	}
	mil = getvar(V_MILIT, (char *)&fsect, EF_SECTOR);
	shell = getvar(V_SHELL, (char *)&fsect, EF_SECTOR);
	gun = getvar(V_GUN, (char *)&fsect, EF_SECTOR);
	fort = (fsect.sct_type == SCT_FORTR);
	fx = fsect.sct_x;
	fy = fsect.sct_y;
#ifdef ELEVATIONS
	flev = max(0,fsect.sct_elev);
#else
	flev = (fsect.sct_type==SCT_MOUNT) ? 1 : 0;
#endif
	milneeded = 5;
	attacker = land_based;

    } else {

	/* FIRE FROM SEA */

	if ((fshipno=atoi(fbuf))<0 || !getship(fshipno,&fship))
	    return RET_SYN;
	if (fship.shp_own != cnum) {
	    pr(fmt("%s is not yours\n",shipas(&fship)));
	    return RET_FAIL;
	}
	if (fship.shp_aboard >= 0) {
	    pr(fmt("%s is being transported\n",shipas(&fship)));
	    return RET_FAIL;
	}
	if (issub(&fship) && fship.shp_depth > 0) {
	    pr(fmt("%s is submerged\n",shipas(&fship,0)));
	    return RET_FAIL;
	}
	if (fship.shp_effic < 60) {
	    pr(fmt("%s is crippled (%d%%)\n",
		   shipas(&fship,0), fship.shp_effic));
	    return RET_FAIL;
	}
	if (mchr[fship.shp_type].m_frnge==0) {
	    pr(fmt("%s not equipped for firing\n",shipas(&fship,0)));
	    return RET_FAIL;
	}
	mil = getvar(V_MILIT, (char *)&fship, EF_SHIP);
	shell = getvar(V_SHELL, (char *)&fship, EF_SHIP);
	gun = getvar(V_GUN, (char *)&fship, EF_SHIP);
	gun = min(gun, mchr[fship.shp_type].m_glim);
	fx = fship.shp_x;
	fy = fship.shp_y;
#ifdef ELEVATIONS
	if (getsect(fx,fy,&fsect)) flev = max(0,fsect.sct_elev);
	else flev = 0;
#else
	flev = 0;
#endif
	milneeded = 1;
	attacker = ship_based;

    }

    /*
    **  Qualify the attack and prepare the guns for firing
    */

    if (!gun) { 
	pr("Insufficient arms.\n"); 
	return RET_FAIL; 
    }

    if (!shell) {
	pr("Klick!     ...\n"    ); 
	return RET_FAIL;
    }

    if (mil<milneeded) { 
	pr("Not enough military for firing crew.\n"); 
	return RET_FAIL; 
    }

    if (attacker == ship_based) {
	
	/* FIRE FROM SEA */

	range = techfact(fship.shp_tech, mchr[fship.shp_type].m_frnge/2.0);
	pr(fmt("range is %.2f\n", range));
	if (tsub) {
	    shots = 2;
	    if ((mchr[fship.shp_type].m_flags & M_DCH) == 0) {
		pr(fmt("A %s can't drop depth charges!\n",
		       mchr[fship.shp_type].m_name));
		return RET_FAIL;
	    }
	    if (fx!=x || fy!=y) {
		pr("Y-guns can't shoot that far!\n");
		return RET_FAIL;
	    }
	    if (shell < 2) {
		pr("Not enough shells for depth charge!\n");
		return RET_FAIL;
	    }
	} else {
	    ptr = getstarg(argp[3], "How many guns (return for max)? ");
	    shots = (!ptr || !*ptr) ? -1 : atoi(ptr);
	    gun = min(gun, shell);
	    if (isship(&fship)) gun = min(gun, mil / 2);
	    gun = max(gun, 1);
	    if (shots > gun || shots < 0)
		shots = gun;
	    else if (shots == 0) {
		pr("No shots fired.\n");
		return RET_FAIL;
	    }
	}
	guneff = (int) seagun(fship.shp_type, fship.shp_effic, shots);
	def = seadef(fship.shp_type);
	shell -= shots;
	putvar(V_SHELL, shell, (char *)&fship, EF_SHIP);
	fship.shp_mobil = max(fship.shp_mobil-fire_mob_cost, -100);

    } else {
	
	/* FIRE FROM LAND*/

#ifdef ANYCANFIRE
	if (!fort)  switch(fsect.sct_type) {
	case SCT_SANCT:
	case SCT_WASTE:
	case SCT_BSPAN:
	case SCT_WAREH:
	    pr(fmt("Can't fire from a %s!\n",
		   dchr[fsect.sct_type].d_name));
	    return RET_FAIL;
	}
#else
	if (!fort) {
	    pr("Not a fortress.\n");
	    return RET_FAIL;
	}
#endif
	if (target==ship_based && tsub && vship.shp_depth>0) {
	    pr("Target not sighted!\n");
	    return RET_FAIL;
	}

	switch (fsect.sct_type) {
	case SCT_FORTR:
	    guneff = landgun((int)fsect.sct_effic); /* 20..29 */
	    range = tfact(cnum, (double)min(gun,7)); 
	    break;
	case SCT_MOUNT:
	    guneff = 25.0;
	    range = 0.90;
	    break;
	default:
	    guneff = landgun((int)fsect.sct_effic); /* 20..29 */
	    range = 0.90;
	    break;
	}

#ifdef ELEVATIONS
	guneff += (double)(flev-vlev)/10.0;
	range  += (double)(flev-vlev)/40.0;
#else
	guneff += (double)(flev-vlev)*5.0;
	range  += (double)(flev-vlev)*0.5;
#endif
	if (range < 0.5) range = 0.5;
	if (guneff < 10) guneff = 10;

	shots = 1;
#ifdef ELEVATIONS
	pr(fmt("firing %shill from elev %d to %d\n",
	       (flev>vlev) ? "down" : "up",flev,vlev));
#endif
	pr(fmt("range is %.2f\n", range));
	def = landdef((int)fsect.sct_type);
	putvar(V_SHELL, --shell, (char *)&fsect, EF_SECTOR);
    }

    /*
    **  Fire the guns
    */

    trange = mapdist(x, y, fx, fy);
    if ((trange > range) || (trange > 7)) {
	if (target==land_based) {
	    pr("Target out of range.  Thud.\n");
	} else {
	    pr(fmt("Target %s out of range.  %s.\n",
		   class[mchr[vship.shp_type].m_class].name,
		   isship(&vship) ? "Splash" : "Thud" ));
	}	
	return RET_FAIL; /* changes not written out */
    }
    if (!trechk(cnum, vict, target == land_based ? LANFIR : SEAFIR))
	return RET_FAIL;
    if (tsub) {
	pr("Kawhomp!!!\07\n");
    } else {
	pr("Kaboom!!!\07\n");
	prb = (double)trange / range;
	prb *= prb;
	if (chance(prb)) {
	    pr(fmt("Wind deflects shell%s.\n", splur(shots)));
	    guneff = (int)
		(guneff * (random() & 3) ? 0.5 : 1.0 - prb);
	}
    }
    dam = shelldam((double)guneff, vdef);
    btused = roundavg(guneff / 20.0);
    switch (target) {
    case land_based:
	nreport(cnum, N_SCT_SHELL, vict, 1);
	sectdamage(&vsect, dam);
	if (vict && vict != cnum)
	    wu(0, vict,
	       fmt("Country #%d shelled sector %s for %d%% damage",
		   cnum, xyas(x, y, vict), dam));
	pr(fmt("Shell%s hit sector %s for %d%% damage.\n",
	       splur(shots), xyas(x, y, cnum), dam));
	break ;
    case ship_based:
	switch (vehicleclass(&vship)) {
	case CLASS_TANK: nreport(cnum, N_TNK_SHELL, vict, 1); break;
	case CLASS_BALLOON: nreport(cnum, N_BAL_SHELL, vict, 1); break;
	case CLASS_TRAIN: nreport(cnum, N_TRN_SHELL, vict, 1); break;
	default:   nreport(cnum, N_SHP_SHELL, vict, 1); break;
	}
    default:
	shipdamage(&vship, dam);
	if (vict)
	    wu(0, vict,
	       fmt("Country #%d %s %s for %d%% damage",
		   cnum,
		   tsub ? "depthcharged" : "shelled",
		   shipas(&vship,0),dam));
	if (vship.shp_effic > 20)
	    pr(fmt("%s%s hit %s for %d%% damage.\n",
		   tsub ? "Depthcharge" : "Shell",
		   splur(shots), shipas(&vship,0), dam));
	break ;
    }

    /*
    **  Stoke up defensive fire
    */

    if (target==land_based) {
	if (dam = fortdef(&vsect, def, fx, fy)) {
	    switch (attacker) {
	    case land_based:
		sectdamage(&fsect, dam);
		(void)strcpy(buf, fmt("sector %s",xyas(fx, fy, vict)));
		break ;
	    default:
		shipdamage(&fship, dam);
		(void)strcpy(buf, fmt("%s #%d",
				      mchr[fship.shp_type].m_name,fshipno));
		break ;
	    }
	    if (vict)
		wu(0, vict,
		   fmt("In defence, sector %s did %d%% damage to %s",
		       xyas(x, y, vict), dam, buf));
	}
    } else {
	if (dam = shipdef(vict, def, fx, fy)) {
	    switch (attacker) {
	    case land_based:
		sectdamage(&fsect, dam);
		if (vict)
		    wu(0, vict,
		       fmt("Return fire damaged sector %s %d%%",
			   xyas(fx, fy, vict), dam));
		break ;
	    default:
		if (vict)
		    wu(0, vict,
		       fmt("Return fire damaged %s #%d %d%%",
			   mchr[fship.shp_type].m_name,
			   fshipno, dam));
		shipdamage(&fship, dam);
		break ;
	    }
	}
    }
    if (dam = defdef(vict, def, fx, fy)) {
	switch (attacker) {
	case land_based:
	    sectdamage(&fsect, dam);
	    (void)strcpy(buf, fmt("sector %s",xyas(fx, fy, vict)));
	    break ;
	default:
	    shipdamage(&fship, dam);
	    (void)strcpy(buf, fmt("%s #%d",
				  mchr[fship.shp_type].m_name,fshipno));
	    break ;
	}
	if (vict)
	    wu(0, vict,
	       fmt("Fortress(es) defending %s did %d%% damage to %s",
		   xyas(x, y, vict), dam, buf));
    }
    if (target==land_based) putsect(&vsect); else putship(vshipno, &vship);
    if (attacker==land_based) putsect(&fsect); else putship(fshipno, &fship);
    NAT_DELTA(nat_btu, cnum, -btused);
    return RET_OK;
}

static int fire_issector(arg)
char *arg;
{
    char c;
    while (c = *arg++)
	if (!isdigit(c) && !isspace(c)) return 1;
    return 0;
}
