#ifndef lint
static char *RCSid = "$Header: mark.c,v 1.1 89/12/14 08:20:09 jay Exp $";
#endif /* not lint */

/*
 * mark.c
 *
 * World market report for commodities.
 * displays only the "best prices" for all the
 * commodities available for trading.
 *
 * Dave Pare, 1986
 */

#include "misc.h"
#include "var.h"
#include "nat.h"
#include "item.h"
#include "deity.h"

#ifdef TRADECENTERS

/* ________________________________________________________________
**|                                                                |
**|             T R A D E   C E N T E R   M A R K E T              |
**|________________________________________________________________|
*/

#include "xy.h"
#include "file.h"
#include "sect.h"
#include "nsc.h"
#include "shop.h"

int numtcs;
struct {
    natid own;
    tradecenter *ptr;
    int stock[I_MAX+1];
} tc[MAXTRADECENTERS];

mark()
{
    extern char *argp[];
    int i,j,n,m,avail,nat;
    char *p,*t;
    struct ichrstr *ip;
    struct sctstr sect;
    struct nstr_sect nstr;
    tradecenter shop,*tcp;
    float price;

    if (argp[1] && argp[2] && !strcmp(argp[1],"debug")) {
	if (!strcmp(argp[2],"on")) dpr(1); else
	if (!strcmp(argp[2],"off")) dpr(0);
	return RET_OK;
    }

    p = argp[1];
    if (!p || !*p) p = "*";
    if ((nat=natorsectarg(p,0)) == -1)
	return RET_SYN;
    
    /* show tradecenters in a specific sector or region of sectors */
    if (nat == -3) {
	if (!snxtsct(&nstr,p)) return RET_SYN;
	for (numtcs=n=0; nxtsct(&nstr,&sect); ) {
	    if (sect.sct_type != SCT_TRADE) continue;
	    if (sect.sct_newtype != SCT_TRADE) continue;
	    lockshop();
	    if (!getshop(sect.sct_x,sect.sct_y,&shop)) continue;
	    if (goodshop(&shop)) {
		getsect(shop.x,shop.y,&sect);
		if ( sect.sct_own==cnum || shopstatus(&shop,cnum,2) ) {
		    if (tcp=(tradecenter *)malloc(sizeof(tradecenter))) {
			int item[I_MAX+1];
			getvec(VT_ITEM, item, (char *)&sect, EF_SECTOR);
			tcp->x = shop.x;
			tcp->y = shop.y;
			for (j=0;j<MAXNOC;j++)
			    tcp->pricemult[j] = shop.pricemult[j];
			for (i=0;i<=I_MAX;i++) {
			    tcp->item[i].buyprice  = shop.item[i].buyprice;
			    tcp->item[i].sellprice = shop.item[i].sellprice;
			    tcp->item[i].lowater   = shop.item[i].lowater;
			    tcp->item[i].hiwater   = shop.item[i].hiwater;
			    tc[numtcs].stock[i] = item[i];
			}
			tc[numtcs].own = sect.sct_own;
			tc[numtcs].ptr = tcp;
			numtcs++;
		    } else {
			logerror("market: malloc failure");
			for (i=0;i<numtcs;i++) free(tc[i].ptr);
			return RET_FAIL;
		    }
		}
	    }
	    unlockshop();
	}
    }

    /* show tradecenters owned by a specific country */
    else {
	lockshop();
	for (numtcs=n=0; shopread(&shop,n); n++) {
	    if (goodshop(&shop)) {
		getsect(shop.x,shop.y,&sect);
		if ((sect.sct_type == SCT_TRADE) &&
		    (sect.sct_own == cnum || shopstatus(&shop,cnum,2)) &&
		    ((nat == -2) || (nat == sect.sct_own)) ) {
		    if (tcp=(tradecenter *)malloc(sizeof(tradecenter))) {
			int item[I_MAX+1];
			getvec(VT_ITEM, item, (char *)&sect, EF_SECTOR);
			tcp->x = shop.x;
			tcp->y = shop.y;
			for (j=0;j<MAXNOC;j++)
			    tcp->pricemult[j] = shop.pricemult[j];
			for (i=0;i<=I_MAX;i++) {
			    tcp->item[i].buyprice  = shop.item[i].buyprice;
			    tcp->item[i].sellprice = shop.item[i].sellprice;
			    tcp->item[i].lowater   = shop.item[i].lowater;
			    tcp->item[i].hiwater   = shop.item[i].hiwater;
			    tc[numtcs].stock[i] = item[i];
			}
			tc[numtcs].own = sect.sct_own;
			tc[numtcs].ptr = tcp;
			numtcs++;
		    } else {
			logerror("market: malloc failure");
			for (i=0;i<numtcs;i++) free(tc[i].ptr);
			return RET_FAIL;
		    }
		}
	    }
	}
	unlockshop();
    }

    pr("\n           -= World Commodity Market Report =-\n");
    pr("                 ");
    prdate();

    for (i=m=0;ichr[i].i_name;i++) {
	ip = &ichr[i];
	if (ip->i_sell == 0) continue;
	if (t=argp[2]) {
	    while (*t && *t!=ip->i_mnem) t++;
	    if (!*t) continue;
	}
	for (n=0;n<numtcs;n++) {
	    if (tc[n].ptr->item[i].buyprice<0.0 &&
		tc[n].ptr->item[i].sellprice<0.0) continue;
	    if (!m++) {
		pr("item          x,y     country   ");
		pr("sell     ask   buy     bid    min have max\n");
	    }
	    pr(fmt("%-10.10s ",ip->i_name));
	    pr(xyfmt("%4d,%-4d ",tc[n].ptr->x,tc[n].ptr->y,cnum));
	    pr(fmt("%-9.9s ",cname(tc[n].own)));
	    
	    /* 
	     **  Report how much the sector can sell.
	     **  (We never let a sector sell so much of
	     **  a commodity that its remaining inventory
	     **  would drop below the `lowater' mark.)
	     */
	    avail = tc[n].stock[i] - tc[n].ptr->item[i].lowater;
	    if (avail < 0) avail = 0;
	    if ((tc[n].own!=cnum && !avail)
		|| (tc[n].ptr->item[i].sellprice<0.0))
		pr(fmt("%14.14s",""));
	    else {
		price = tc[n].ptr->pricemult[cnum] 
		    * tc[n].ptr->item[i].sellprice;
		pr(fmt("%5d%9.2f",avail,price));
	    }
	    
	    /* 
	     **  Report how much the sector can buy.
	     **  (We never let a sector buy so much of
	     **  a commodity that its total inventory
	     **  would exceed the `hiwater' mark.)
	     */
	    
	    avail = tc[n].ptr->item[i].hiwater - tc[n].stock[i];
	    if (avail < 0) avail = 0;
	    if ((tc[n].own!=cnum && !avail) 
		|| (tc[n].ptr->item[i].buyprice<0.0))
		pr(fmt("%14.14s",""));
	    else 
		pr(fmt("%5d%9.2f",avail,tc[n].ptr->item[i].buyprice));
	    
	    if (cnum == tc[n].own) {
		pr(fmt("  %4d%5d%4d",tc[n].ptr->item[i].lowater,
		       tc[n].stock[i],tc[n].ptr->item[i].hiwater));
	    }
	    pr("\n");
	}
    }
    for (i=0;i<numtcs;i++) free(tc[i].ptr);
    return RET_OK;
}

#else

/* ________________________________________________________________
**|                                                                |
**|                 T E L E P O R T   M A R K E T                  |
**|________________________________________________________________|
*/

mark()
{
	extern	char *argp[];
	struct	ichrstr *ip;
	int	product;
	int	sellers;
	natid	seller;
	float	price;
	float	bestprice;
	int	amt;
	int	n;
	natid	cn;

	if (commread() < 0) {
		pr("Unable to read commodity file; get help!\n");
		return RET_SYS;
	}
	if (argp[1] && *argp[1]) {
		ip = whatitem(argp[1], "Commodity: ");
		if (ip == 0)
			return RET_OK;
		if (ip->i_sell == 0) {
			pr(fmt("%s cannot be traded!\n", ip->i_name));
			return RET_OK;
		}
		product = ip->i_vtype & ~VT_TYPE;
		sellers = 0;
		for (cn = 0; cn < MAXNOC; cn++) {
			if ((amt = commamt(cn, product, &price)) == 0)
				continue;
			if (sellers == 0) {
				pr(fmt("Trade report: %s\n", ip->i_name));
				pr(" cnum       country   amount  item price\n");
			}
			pr(fmt("(#%3d) %12.12s %7d %7.2f", cn, cname(cn),
			       amt, price));
			if (cn == cnum && !god)
				pr(" (your own lot)\n");
			else
				pr("\n");
			sellers++;
		}
		if (sellers == 0) {
			pr(fmt("No countries are selling %s\n", ip->i_name));
			return RET_OK;
		}
		return RET_OK;
	}
	pr("      -= World Commodity Market Report =-\n");
	pr("            -= Today's Best Buys =-\n");
	pr("            ");
	prdate();
	pr(" item          cnum      country     qty  $/unit\n");
	bestprice = 0;
	for (ip = &ichr[1]; ip->i_name != 0; ip++) {
		if (ip->i_sell == 0)
			continue;
		seller = 0;
		product = ip->i_vtype & ~VT_TYPE;
		for (cn = 1; cn < MAXNOC; cn++) {
			if (cn == cnum && !god)
				continue;
			if ((n = commamt(cn, product, &price)) <= 0)
				continue;
			if (seller == 0 || (bestprice >= price)) {
				if ((bestprice == price) && (amt >= n))
					continue;
				bestprice = price;
				amt = n;
				seller = cn;
			}
		}
		if (seller)
			pr(fmt("%-12.12s (#%3d) %12.12s %7d %7.2f\n",
			       ip->i_name, seller, cname(seller), amt,
			       bestprice));
	}
	return RET_OK;
}

#endif
