#ifndef lint
static char *RCSid = "$Header: para.c,v 1.1 89/12/14 08:20:28 jay Exp $";
#endif

/*
 * para.c
 *
 * Drop paratroops onto a sector
 *
 * Dave Pare, 1986
 */

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "ship.h"
#include "item.h"
#include "plane.h"
#include "nuke.h"
#include "xy.h"
#include "nsc.h"
#include "news.h"
#include "file.h"
#include "nat.h"
#include "path.h"

extern double paradef_mult;  /* defense bonus scalar */
extern double paradef_base;  /* defense bonus base */
extern double paradef_minb;  /* defense minimum bonus */
extern double paradef_capi;  /* capitol defense multiplier */
extern double paradef_fort;  /* fort defense multiplier */

static	paradrop();

para()
{
	extern	char *argp[];
	char	*p;
	int	mission_flags;
	coord	tx, ty;
	coord	ax, ay;
	int	ap_to_target;
	char	flightpath[256];
	struct	nstr_item ni_bomb;
	struct	nstr_item ni_esc;
	coord	x, y;
	struct	sctstr target;
	struct	qelem bomb_list;
	struct	qelem esc_list;
	int	wantflags;
	struct	sctstr ap_sect;

	wantflags = 0;
	if (!snxtitem(&ni_bomb, EF_PLANE, argp[1]))	return RET_SYN;
	if (!snxtitem(&ni_esc, EF_PLANE, argp[2]))	pr("No escorts...\n");
	if (!getassemblypoint(argp[1],argp[3],&x,&y))	return RET_SYN;
	if (!getsect(x, y, &ap_sect))			return RET_SYN;

#ifndef ASSEMBLEANYWHERE
	if (ap_sect.sct_own && ap_sect.sct_own != cnum) {
		pr("Assembly point not owned by you!\n");
		return RET_SYN;
	}
#endif

	p = getpath(argp[4], ax=x, ay=y, 0, 0, 0);
	if (!p || !*p) return RET_SYN;
	(void)strcpy(flightpath, p);
	tx = ax;
	ty = ay;
	(void) pathtoxy(flightpath, &tx, &ty, fcost);
	getsect(tx, ty, &target);
	if (target.sct_own == cnum) {
		/* we *could* just drop them off .. */
		pr(fmt("You already own %s\n", xyas(tx, ty, cnum)));
		return RET_SYN;
	}
	pr(fmt("LZ is %s\n", xyas(tx, ty, cnum)));
	ap_to_target = strlen(flightpath);
	pr(fmt("range to target is %d\n", ap_to_target));
	/*
	 * select planes within range
	 */
	pln_sel(&ni_bomb, &bomb_list, &ap_sect, ap_to_target,
		2, wantflags, P_M|P_O, "");
	pln_sel(&ni_esc, &esc_list, &ap_sect, ap_to_target,
		2, P_F|wantflags, P_M|P_O, "");
	/*
	 * now arm and equip the bombers, transports, whatever.
	 * tech is stored in high 16 bits of mission_flags.
	 * yuck.
	 */
	mission_flags = 0x7fff0000;
	mission_flags |= P_X;		/* stealth (shhh) */
	mission_flags = pln_arm(&bomb_list, 'a', &ichr[I_MILIT],
		0, mission_flags);
	if (QEMPTY(&bomb_list)) {
		pr("No planes could be equipped for the mission.\n");
		return RET_FAIL;
	}
	mission_flags = pln_arm(&esc_list, 'a', &ichr[I_MILIT],
		P_F, mission_flags);
	ac_encounter(&bomb_list, &esc_list, ax, ay, flightpath, mission_flags);
	if (QEMPTY(&bomb_list)) {
		pr(fmt("No planes got through fighter defenses\n"));
	} else {
		getsect(tx, ty, &target);
		paradrop(&bomb_list, &target);
		putsect(&target);
	}
	pln_put(&bomb_list);
	pln_put(&esc_list);
	return RET_OK;
}

static paradrop(list, target)
struct	qelem *list;
struct	sctstr *target;
{
    extern	int btused;
    register struct plist *plp;
    int	a_mil,i_mil,i;
    int	a_cas;
    int	d_mil;
    int	d_cas;
    int	d_she,i_she;
    int	d_gun;
    int	success;
    double	odds;
    double	flakodds;
    double	shuffle;
    struct	qelem *qp;
    double	d_bonus;

    d_mil = getvar(V_MILIT, (char *)target, EF_SECTOR);
    d_gun = getvar(V_GUN, (char *)target, EF_SECTOR);
    d_she = getvar(V_SHELL, (char *)target, EF_SECTOR);

    a_mil = a_cas = d_cas = success = 0;
    for (qp = list->q_forw; qp != list; qp = qp->q_forw) {
	plp = (struct plist *) qp;
	if (plp->pcp->pl_flags & (P_V|P_C))
	    a_mil += plp->misc;
    }

    dpr(fmt("paradrop: a_mil=%d, d_mil=%d, d_gun=%d, d_she=%d",
	    a_mil,d_mil,d_gun,d_she));
    
    /* I juggled the following code around a bit to make
       it possible, although very difficult, to air assault
       capitols and fortresses.  9/20/90 bf@hp */

    d_bonus = paradef_base + paradef_mult*dchr[target->sct_type].d_dstr;
    
    dpr(fmt("paradrop: target %d d_bonus initially %.2f",
	    target->sct_type,d_bonus));
    
    switch (target->sct_type) {
    case SCT_MOUNT:
    case SCT_WATER:
    case SCT_WASTE:
	a_cas = a_mil; /* such an attack *never* works */
	goto fail;
    case SCT_CAPIT:
	d_bonus *= paradef_capi;
	break;
    case SCT_FORTR:
	d_bonus *= paradef_fort;
	break;
    }

    if (d_bonus < paradef_minb) d_bonus = paradef_minb;
    d_bonus *= target->sct_effic/100.0;

    dpr(fmt("paradrop: d_bonus after adjustments = %.2f", d_bonus));

    /* sanity check */
    if (d_bonus <= 0.0)   {
	d_bonus = 1.0;
	d_mil = 0;
    }

    if (a_mil <= 0)  odds = 0.0;
    else if (d_mil <= 0)  odds = 1.0;
    else  	          odds = a_mil / (d_bonus * (d_mil + a_mil));

    if (d_mil == 0 && a_mil > 0)  {
	dpr(fmt("paradrop: no defenders"));
	pr("There are no defenders\n");
	success++; 
    } else {
	dpr(fmt("paradrop: odds are %.2f%%",100*odds));
	pr(fmt("Your odds are %.2f%%\n", 100*odds));
    }

    /*
    **  This makes air assaults more interesting.
    **  If mil on the ground have guns and shells,
    **  there's a good chance they will flak the
    **  paratroopers.  9/20/90 bf@hp
    */

    i_she = d_she;
    i_mil = a_mil;
    while (!success && a_mil>0 && d_mil>0
	   && d_gun && d_she 
	   && chance((d_she>(d_gun*2)) ? 0.75 
		     : ((d_she>d_gun)  ? 0.66 
		     : 0.50))) {
	flakodds = ((d_gun>d_she) ? d_she : d_gun)/16.0;
	flakodds += a_mil/64.0;
	if (i_she == d_she) pr("Ground artillery firing!\n");
	pr("*"); 
	d_she--;
	if (chance(flakodds)) {
	    i = roll(4);
	    a_mil -= i;
	    while (i--) pr("@");
	}
    }
    if (i_she != d_she) { /* if shots were fired, recalculate odds */
	if (a_mil <= 0)	     odds = 0.0;
	else if (d_mil <= 0) odds = 1.0;
	else  	   	     odds = a_mil / (d_bonus * (d_mil + a_mil));
	pr(fmt("%d paratroops hit, %d remain\n",i_mil-a_mil,a_mil));
	pr(fmt("Your odds are now %.2f%%\n", 100*odds));
	putvar(V_SHELL, d_she, (char *)target, EF_SECTOR);
	dpr(fmt("paradrop: postflak odds are %.2f%%",100*odds));
    }

    /*
    **  We now resume our regularly scheduled code ...
    */

    while (!success && a_mil > 0 && d_mil > 0) {
	if (chance(odds)) {
	    pr("!");
	    d_cas++;
	    d_mil--;
	    if (d_mil <= 0)
		success++;
	} else {
	    pr("@");
	    a_mil--;
	    a_cas++;
	}
    }

fail:

    if (target->sct_own) {
	wu(0, target->sct_own,
	   fmt("Country #%d lost %d air-assaulting %ssector %s",
	       cnum, a_cas, success ? "(and taking) " : "",
	       xyas(target->sct_x, target->sct_y, target->sct_own)));
	wu(0, target->sct_own, 
	   fmt("  %s casualties: %d",cname(target->sct_own),d_cas));
    }
    if (success) {
	pr(fmt("You have taken sector %s\n",
	       xyas(target->sct_x, target->sct_y, cnum)));
	nreport(cnum, N_WON_SECT, target->sct_own, 1);
	if (target->sct_type == SCT_CAPIT)
	    caploss(target, target->sct_own,
		    "which is also %s's capital!\n");
	a_mil = takeover(target, a_mil);
	target->sct_mobil = 0;
	putvar(V_MILIT, a_mil, (char *)target, EF_SECTOR);
    } else {
	pr("You have been defeated!\n");
	nreport(cnum, N_SCT_LOSE, target->sct_own, 1);
	putvar(V_MILIT, d_mil, (char *)target, EF_SECTOR);
    }
    if (a_cas > 0 || d_cas > 0) {
	pr(fmt("Casualties :\nYours... %d\n", a_cas));
	pr(fmt("Theirs.. %d\n", d_cas));
	shuffle = 2 * (a_cas + d_cas) * 0.15;
	pr(fmt("Papershuffling ... %.1f B.T.U\n", shuffle));
	NAT_DELTA(nat_btu, cnum, -roundavg(shuffle));
    }
}
