#ifndef lint
static char *RCSid = "$Header: pr.c,v 1.1 89/12/14 13:28:30 jay Exp $";
#endif

/*
 * pr.c
 *
 * The pr routine historically arranged for nonbuffered i/o
 * because stdio didn't used to automatically flush stdout before
 * it read something from stdin.  Now pr() prepends an "output id"
 * in front of each line of text, informing the user interface
 * what sort of item it is seeing; prompt, noecho prompt,
 * more input data, etc.
 *
 * Dave Pare, 1986, 1989
 */

#include <stdio.h>
#include <ctype.h>
#include <varargs.h>
#include <fcntl.h>
#include "deity.h"
#include "proto.h"
#include "misc.h"
#include "nat.h"
#include "sailio.h"
#include "class.h"

static	int bol = 1;

pr(buf)
char *buf;
{
    register char *bp;
    register int c;
    register int local_bol;
    local_bol = bol;
    bp = buf;
    while (c = *bp++) {
	if (local_bol) {
	    (void) outid(C_DATA);
	    local_bol = 0;
	}
	(void) putc(c, stdout);
	if (c == '\n')
	    local_bol++;
    }
    bol = local_bol;
}

/*
**  Like pr(), but sends via telegram if recipient isn't 'cnum'.
*/

prc(c1,buf)
char *buf;
natid c1;
{
    if (c1==cnum) pr(buf); else wu(0,c1,buf);
}


/*
**  Like pr(), but sends same message to two countries, via telegram
**  if necessary.  Note: the second copy isn't sent if the recipient
*   is 0 (POGO).
*/

prcc(c1,c2,buf)
char *buf;
natid c1,c2;
{
    prc(c1,buf);
    if (c2 && c2!=c1) prc(c2,buf);
}

/*
**  Highlighted characters have hex 80 or'ed in
**  with them to designate their highlightedness
*/

pr_hilite(buf)
char *buf;
{
    register char *bp;
    register char c;
    register int local_bol;
    local_bol = bol;
    for (bp=buf; c = *bp; bp++) {
	if (local_bol)
	    (void) outid(C_DATA);
	local_bol = 0;
	if (isprint(c))
	    c |= 0x80;
	(void) putc(c, stdout);
	if (c == '\n')
	    local_bol++;
    }
    bol = local_bol;
}

/* 
**  Output 3 digit code plus space
*/

outid(n)
int n;
{
    char c;
    if (n > 16) {
	logerror("outid: %d not valid code\n", n);
	return;
    }
    if (n >= 10) 
	c = 'a' - 10 + n;
    else 
	c = '0' + n;
    putc(c, stdout);
    putc(' ', stdout);
}

prredir(redir)
char *redir;
{
    if (*redir == '>') outid(C_REDIR);
    else outid(C_PIPE);
    printf("%s\n", redir);
}

prexec(file)
char *file;
{
    outid(C_EXECUTE);
    printf("%s\n", file);
}

/*
**  Informs user interface that it should immediately
**  flush it's stdout after receiving this line.  This
**  is probably a prompt.
*/
prflush(buf)
char *buf;
{
    register char *bp;
    register int c;
    if (!bol) (void) putc('\n', stdout);
    (void) outid(C_FLUSH);
    for (bp=buf; c = *bp; bp++) {
	(void) putc(c, stdout);
	if (c == '\n') (void) outid(C_FLUSH);
    }
    if (c != '\n') {
	(void) putc('\n', stdout);
	bol++;
    }
    if (fflush(stdout) < 0) {
	/*gencore();*/
	tm_exit(0);
    }
}

prprompt(min, btu)
int min,btu;
{
    outid(C_PROMPT);
    printf("%d %d\n", min, btu);
}

showvers(vers)
int vers;
{
    (void) outid(C_INIT);
    printf("%d\n", vers);
}

int prmptrd(prompt, str, size)
char *prompt;
char *str;
int size;
{
    int	r;
    prflush(prompt);
    if ((r = recvclient(str, size)) <= 0) return r;
    if (*str == 0) return 1;
    return strlen(str);
}

prdate()
{
    extern char *ctime();
    long now;
    (void) time(&now);
    pr(ctime(&now));
}

prest(pcterr)
int pcterr;
{
    if (pcterr>0 && pcterr<=100)
	pr(fmt("Report accuracy is %d%%\n",100-pcterr));
}

prvcheader(vc,more)
int vc;
char *more;
{
    /* 
    **  Print a vehicle id/type header for format "%4d %-16.16s".
    **  Include a "%3d " owner field at the start if user is god.
    */
    switch(vc) {
    case CLASS_TANK:    pr("tnk#   tank type     "); break;
    case CLASS_BALLOON: pr("bal#   balloon type  "); break;
    case CLASS_TRAIN:   pr("trn#   train type    "); break;
    case CLASS_SHIP:    pr("shp#   ship type     "); break;
    default:	        pr("veh#   vehicle type  "); break;
    }
    if (more) pr(more);
}

summarize(argp,num,name)
char *argp,*name;
int num;
{
    char buf[30],*s;
    strcpy(buf,name);
    s = buf+strlen(buf)-1;
    if (num != 1) {
	if (*s == 'y') strcpy(s,"ies");
	else strcat(s,"s");
    }
    if (num == 0) {
	pr(fmt("%s: No %s\n", (argp) ? argp : "",buf));
	return RET_FAIL;
    }
    pr(fmt("%d %s\n",num,buf));
    return RET_OK;
}

/* ________________________________________________________________
**|                                                                |
**| Debug output processor                                         |
**|________________________________________________________________|
*/
dpr(s,p1,p2,p3,p4,p5,p6,p7,p8,p9) 
char *s,*p1,*p2,*p3,*p4,*p5,*p6,*p7,*p8,*p9;
{
    int i;
    static int dflag = 0;
    static int dindent = 0;
    static FILE *tfp = 0;
    if ((int)s < 2) 
	dflag = (int)s;
    else {
	if (*s == '~') { /* use a specific output file */
	    if (s[1]=='!') {
		if (tfp && tfp>stderr) fclose(tfp);
		tfp = p1 ? ((p1<=stderr) ? (FILE *)p1 : fopen(p1,p2)) : 0;
	    } else {
		if (tfp) {
		    fprintf(tfp,p1,p2,p3,p4,p5,p6,p7,p8,p9);
		    fflush(tfp);
		}
	    }
	}
	else { /* use our usual 'pr' output */
	    for (;*s=='>' || *s=='<';s++)
		dindent += ((*s=='>') ? 1 : -1);
	    if (dindent < 0) dindent = 0;
	    if (dflag && s && *s) {
		pr("*** "); for (i=0;i<dindent;i++) pr("  "); 
		pr(s); pr("\n");
	    }
	}
    }
}

