#ifndef lint
static char *RCSid = "$Header: main.c,v 1.1 89/12/14 13:30:09 jay Exp $";
#endif

/*
 * main.c
 *
 * main command loop and initialization
 * for empire tm server
 *
 * Dave Pare, 1986
 */
#include "misc.h"
#include "tm.h"
#include "sailio.h"
#include "user.h"
#include "bit.h"
#include "file.h"

#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/un.h>
#include <signal.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#ifndef SIGCHLD
#define SIGCHLD SIGCLD
#endif

extern	char serverport[];

static	int max_fd;
static	int expire;
static	int done;

int	(*leprfunc)();		/* logerror pr function, == NULL */

char	*program = "tm";

alarmhandler()
{
	expire++;
#ifdef sys5
	(void) signal(SIGALRM, alarmhandler);
#endif
}

child()
{
	done++;
#ifdef sys5
#ifdef hpux
	/* HP-UX can't reset signal handler until the wait(2) is done */
#else
	(void) signal(SIGCHLD, child);
#endif
#endif
}

cleanup(code)
	int	code;
{
	(void) unlink(serverport);
	exit(code);
}

main()
{
	int	child();
	int	catch();
	extern	int errno;
	register int n;
	struct	sockaddr_un un;
	struct	user *up;
	bit_fdmask savemask;
	bit_fdmask mask;
	long	now;
	int	s;
	int	reserve;
	int	fd;
	int	hour[2];
	struct	timeval tv;

#ifndef DEBUG
	disassoc();
#endif
	time(&now);
	srandom(now);
	setfdtablesize(32, 256);
	initfiles();
	(void) signal(SIGALRM, alarmhandler);
	(void) signal(SIGCHLD, child);
	(void) signal(SIGTERM, cleanup);
#ifndef DEBUG
	(void) signal(SIGPIPE, SIG_IGN);
	(void) signal(SIGBUS, catch);
	(void) signal(SIGILL, catch);
	(void) signal(SIGFPE, catch);
	(void) signal(SIGSEGV, catch);
#endif
	(void) gamehours(now, hour);
	initlock();
	schedule_update();
	strcpy(un.sun_path, serverport);
	un.sun_family = AF_UNIX;
	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
		logerror("unix socket create");
		exit(1);
	}
	if (bind(s, (struct sockaddr *) &un, strlen(un.sun_path)+2) < 0) {
		logerror("another %s running?  If not, rm %s and retry",
			program, un.sun_path);
		exit(1);
	}
	if (listen(s, 5) < 0) {
		logerror("unix socket listen");
		cleanup(1);
	}
	savemask = bit_newfdmask();
	mask = bit_newfdmask();
	BIT_SETB(s, savemask);
	max_fd = s;
	logerror("tm server started, pid %d", getpid());
	/* squirrel away file descriptors "for later" */
	reserve = getfdtablesize() - 5;
	while (1) {
		if (expire) {
			execute();
			expire = 0;
		} else if (done) {
			exec_done();
			done = 0;
		}
		bit_copy(savemask, mask);
		tv.tv_sec = 60*15;
		tv.tv_usec = 0;
		if (select(max_fd+1, mask, 0, 0, &tv) < 0) {
			if (errno == EINTR) {
				/* a signal! */
				errno = 0;
				continue;
			}
			logerror("PANIC: main loop select");
			cleanup(1);
		}
		if (BIT_ISSETB(s, mask)) {
			if ((up = newuser(s, reserve)) == 0) {
				logerror("newuser failed");
				continue;
			}
			/*logerror("new user");*/
			BIT_SETB(up->u_iop->fd, savemask);
			BIT_CLRB(s, mask);
			if (max_fd < up->u_iop->fd)
				max_fd = up->u_iop->fd;
		}
		while ((fd = bit_fd(mask)) >= 0) {
			if ((up = userbyfd(fd)) == 0) {
				logerror("cannot find user to match fd %d", fd);
				(void) close(fd);
				BIT_CLRB(fd, savemask);
			} else if (service(up)) {
				/*logerror("user leaving");*/
				if (fd == max_fd)
					max_fd--;
				BIT_CLRB(fd, savemask);
			}
			BIT_CLRB(fd, mask);
		}
		for (n=0,up=users; n < maxusers; n++,up++) {
			if (up->u_iop == 0)
				continue;
			if (sio_output(up->u_iop)) {
				sflush(up->u_iop);
			}
		}
		/*
		 * flush delayed writes
		 */
		dw_flush();
	}
}

catch(sig)
	int	sig;
{
	logerror("trapped signal %d", sig);
	(void)signal(SIGIOT, SIG_DFL);		/* For Sun */
	(void)signal(SIGILL, SIG_DFL);		/* For VAX */
	(void)abort();
}
