#ifndef lint
static char *RCSid = "$Header: ship.c,v 1.1 89/12/14 13:30:56 jay Exp $";
#endif

/*
 * ship.c
 *
 * do "production" for ships -- make more efficient,
 * charge for military, etc.
 *
 * from PSL Empire, 1985
 * and Dave Pare, 1986
 */

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "nat.h"
#include "ship.h"
#include "var.h"
#include "news.h"
#include "file.h"
#include "product.h"
#include "loc.h"

#define VERBOSE

double basemobcost();

int
prod_ship(etus)
	int	etus;
{
    register struct shpstr *sp;
    extern long sea_money[MAXNOC];
    struct natstr *np;
    int	n,start_money;
    
    for (n=0; sp = getshipp(n); n++) if (sp->shp_own) {
	np = getnatp(sp->shp_own);
/*	if ((np->nat_stat & STAT_NORM) == 0) continue; */
	start_money = np->nat_money;
	upd_ship(sp, n, etus, np);
	sea_money[sp->shp_own] += np->nat_money - start_money;
    }
}

autonav_ship(etus)
int etus;
{
    register struct shpstr *sp, *tp;
    struct locstr { coord x,y; } *loc;
    char *did,*s,convoy;
    int	i,n,maxship,slowestship;
    double maxmobcost,thismob;
    coord savex, savey;
    
    maxship = ef_nelem(EF_SHIP);
    loc = (struct locstr *)malloc(maxship*sizeof(struct locstr));
    did = (char *)malloc(maxship);
    bzero((char *)did, maxship);

    /* autonav ship convoys and note new positions */
    while (1) {

	/* 
	**  Always starting from the beginning, scan the ship list 
	**  looking for next autonaving ship that belongs to a convoy
	**  and has not yet been processed.
	*/

	for (n=0; sp=getshipp(n); n++) {
	    if (did[n]) continue;
	    if (!sp->shp_own || sp->shp_aboard>=0 ||
		mchr[sp->shp_type].m_class != CLASS_SHIP) continue;
	    if ((sp->shp_autonav & AN_AUTONAV) 
		&& !(sp->shp_autonav & AN_STANDBY)
		&& (sp->shp_convoy != ' ')) break;
	}

	/* 
	**  If none left or we've done them all, exit the `while (1)'.
	*/

	if (!sp || (n>maxship)) break;

	/* 
	**  This is the first ship of a yet-unprocessed convoy.
	**  Being the first, it's also the slowest we've seen so far,
	**  so remember that along with its mobility and convoy ID.
	*/

	convoy = sp->shp_convoy;
	slowestship = n;
	maxmobcost = basemobcost(sp);

#ifdef VERBOSE
	wu(0,0,fmt("*** CONVOY '%c' ***\n",convoy));
	wu(0,0,fmt("%s mobcost=%.3f, initial choice '%c' leader\n",
		   shipasc(tp,LOS,0),maxmobcost,convoy));
#endif

	/* 
	**  Scan remainder of ship list for other yet-unprocessed ships
	**  that are autonaving and belong to the same convoy.  Whenever 
	**  we find one, check to see if it's slower than the last slowest 
	**  ship we encountered.  If the new ship is slower, remember it 
	**  along with its mobility.
	*/
	   
	for (n++; tp=getshipp(n); n++) {
	    if (did[n]) continue;
	    if (!tp->shp_own || tp->shp_aboard>=0 ||
		mchr[tp->shp_type].m_class != CLASS_SHIP) continue;
	    if ((tp->shp_autonav & AN_AUTONAV) 
		&& !(tp->shp_autonav & AN_STANDBY)
		&& (tp->shp_convoy == convoy)) {
		thismob = basemobcost(tp);
		if (thismob > maxmobcost) {
#ifdef VERBOSE
		    wu(0,0,
		       fmt("%s mobcost=%.3f, replaces #%d as '%c' leader\n",
			   shipas(tp,0),thismob,slowestship,convoy));
#endif
		    slowestship = n;
		    maxmobcost = thismob;
#ifdef VERBOSE
		} else {
		    wu(0,0,
		       fmt("%s mobcost=%.3f, convoy '%c' follower\n",
			   shipas(tp,0),thismob,convoy));
#endif
		}
	    }
	}

	/*
	**  We should now know which is the slowest ship in the entire
	**  convoy.  Autonav this pace ship toward its destination, mark 
	**  that we've processed it, and note its new position in the final
	**  `loc[]' array.
	*/

	sp = getshipp(slowestship);
	did[slowestship] = 1;
#ifdef VERBOSE
	wu(0,0,fmt("%s is slowest ship in convoy `%c'\n",
		   shipasc(sp,LOS,0),convoy));
#endif
	if (nav_ship(sp,1)) {
	    loc[slowestship].x = sp->shp_x;
	    loc[slowestship].y = sp->shp_y;

	    wu(0, sp->shp_own,
	       fmt("%s, leading convoy `%c', moved to %s\n",
		   shipas(sp,0),convoy,
		   xyas(sp->shp_x,sp->shp_y,sp->shp_own)));
#ifdef VERBOSE
	    wu(0, 0,
	       fmt("%s, leading convoy `%c', moved to %s\n",
		   shipas(sp,0),convoy,
		   xyas(sp->shp_x,sp->shp_y,sp->shp_own)));
#endif
	    /*
	    **  Now run through the whole ship list again, looking for
	    **  yet-unprocessed autonaving ships that belong to the same 
	    **  convoy as the pace ship that we just moved.
	    */

	    for (n=0; tp=getshipp(n); n++) {
		if (did[n]) continue;
		if (!tp->shp_own || tp->shp_aboard>=0 ||
		    mchr[tp->shp_type].m_class != CLASS_SHIP) continue;
		if ((tp->shp_autonav & AN_AUTONAV) 
		    && !(tp->shp_autonav & AN_STANDBY)
		    && (tp->shp_convoy == convoy)) {

		    /*
		    **  Save this ship's actual autonav location, force 
		    **  it to autonav instead toward the pace ship, then
		    **  restore it's actual destination.
		    */

		    savex = tp->shp_destx;
		    savey = tp->shp_desty;
		    if (savex != sp->shp_destx || savey != sp->shp_desty) {
			wu(0, tp->shp_own, 
			   fmt("%s ignoring orders, following convoy `%c'\n",
			       shipasc(tp,LS,tp->shp_own),convoy));
#ifdef VERBOSE
			wu(0, 0,
			   fmt("%s ignoring orders, following convoy `%c'\n",
			       shipasc(tp,LS,0),convoy));
#endif
		    }
		    wu(0, tp->shp_own,
		       fmt("%s tracking convoy `%c'\n",
			   shipasc(tp,LS,tp->shp_own),convoy));
#ifdef VERBOSE
		    wu(0, 0,
		       fmt("%s tracking convoy `%c'\n",
			   shipasc(tp,LS,0),convoy));
#endif
		    tp->shp_destx = sp->shp_x;
		    tp->shp_desty = sp->shp_y;
		    i = (tp->shp_destx==sp->shp_destx) 
			&& (tp->shp_desty==sp->shp_desty);
		    if (nav_ship(tp,i)) {
			did[n] = 1;
			if (tp->shp_x != sp->shp_destx ||
			    tp->shp_y != sp->shp_desty) {
			    tp->shp_autonav |= AN_AUTONAV;
			    tp->shp_autonav &= ~AN_STANDBY;
			}
			if (tp->shp_x != sp->shp_x ||
			    tp->shp_y != sp->shp_y) {
			    wu(0, tp->shp_own, 
			       fmt("%s enroute to %s, moved to %s\n",
				   shipas(tp,0),
				   xyas(tp->shp_destx,tp->shp_desty,
					tp->shp_own),
				   xyas(tp->shp_x,tp->shp_y,tp->shp_own)));
#ifdef VERBOSE
			    wu(0, 0,
			       fmt("%s enroute to %s, moved to %s\n",
				   shipas(tp,0),
				   xyas(tp->shp_destx,tp->shp_desty,0),
				   xyas(tp->shp_x,tp->shp_y,0)));
#endif
			} else {
			    wu(0, tp->shp_own, 
			       fmt("%s joins convoy `%c' in %s\n",
				   shipas(tp,0), convoy,
				   xyas(tp->shp_x,tp->shp_y,tp->shp_own)));
#ifdef VERBOSE
			    wu(0, 0,
			       fmt("%s joins convoy `%c' in %s\n",
				   shipas(tp,0), convoy,
				   xyas(tp->shp_x,tp->shp_y,0)));
#endif
			}
		    }
		    loc[n].x = tp->shp_x;
		    loc[n].y = tp->shp_y;
		    tp->shp_destx = savex;
		    tp->shp_desty = savey;
		}
	    }
	}
    }

    /* autonav all other ships and note their new positions */
    for (n=0; sp = getshipp(n); n++) {
	if (sp->shp_own && !did[n]) nav_ship(sp,1);
	loc[n].x = sp->shp_x;
	loc[n].y = sp->shp_y;
    }
    
    /* insure ferried vehicles tag along */
    for (n=0; sp = getshipp(n); n++)
	if ((sp->shp_own!=0) && (sp->shp_aboard >=0)) {
	    sp->shp_x = loc[sp->shp_aboard].x;
	    sp->shp_y = loc[sp->shp_aboard].y;
	}
    
    free(did);
    free(loc);
}

upd_ship(sp, shipno, etus, np)
	register struct shpstr *sp;
	int	shipno;
	register int etus;
	struct	natstr *np;
{
	extern	long pops[];
	extern  double money_mil;
	extern  int bal_refill_rate;
	extern  int sub_refill_rate;
	struct	sctstr *sectp;
	struct	mchrstr *mp;
	int	vec[I_MAX+1];
	int	cvec[I_MAX+1];
	int	oil_gained;
	int	max_oil;
	int	max_food;
	struct	pchrstr *product;
	char	*resource;
	int	i,n;
	double	leakrate;

	mp = &mchr[sp->shp_type];
	getvec(VT_ITEM, vec, (char *)sp, EF_SHIP);
	if (sp->shp_effic < 20 || !shiprepair(sp, vec, np, etus)) {
		sp->shp_own = 0;
		return;
	}
	if (mp->m_flags & M_OIL) {
		/*
		 * take care of oil production
		 */
		sectp = getsectp(sp->shp_x, sp->shp_y);
		oil_gained = roundavg((vec[I_CIVIL] * etus / 10000.0)
			* sectp->sct_oil);
		vec[I_OIL] += oil_gained;
		max_oil = vl_find(V_OIL, mp->m_vtype, mp->m_vamt, MAXMCHV);
		if (vec[I_OIL] > max_oil)
			vec[I_OIL] = max_oil;
		product = &pchr[P_OIL];
		if (product->p_nrdep != 0 && oil_gained > 0) {
			resource = ((char *) sectp) + product->p_nrndx;
			*resource -=
				roundavg(oil_gained * product->p_nrdep / 100.0);
		}
	} else if (mp->m_flags & M_FOOD) {
		sectp = getsectp(sp->shp_x, sp->shp_y);
		vec[I_FOOD] += ((vec[I_CIVIL] * etus) / 10000.0)
			* sectp->sct_fertil;
	}
	np->nat_money += (int) (etus * vec[I_MILIT] * money_mil);
	if ((n = feed_people(vec, etus)) > 0) {
		wu(0, sp->shp_own, fmt("%d starved on %s #%d\n",
			n, mp->m_name, shipno));
		if (n > 10)
			nreport(sp->shp_own, N_DIE_FAMINE, 0, 1);
	}
	max_food = vl_find(V_FOOD, mp->m_vtype, mp->m_vamt, MAXMCHV);
	if (vec[I_FOOD] > max_food)
		vec[I_FOOD] = max_food;

	/*
	**  Subs: Deplete air; suffocate people if there's not enough.
	**  Balloons: Deplete gas; drop altitude if there's not enough.
	*/
	
	sectp = getsectp(sp->shp_x, sp->shp_y);
	if (sp->shp_depth == 0) { 

	    if (!isballoon(sp)) {

		/* Replenish cabin air supply */
		sp->shp_air += (sub_refill_rate * etus);
		
	    } else if (stypeinlist(class[CLASS_BALLOON].loadplace,
				   sectp->sct_type)) {

		/* Replenish envelope gas supply */
		n = (bal_refill_rate * etus * sectp->sct_effic/100);
		if (sectp->sct_own != sp->shp_own) n >>= 1;
		sp->shp_air += n;
	    }

	    if (sp->shp_air > mchr[sp->shp_type].m_maxair) 
		sp->shp_air = mchr[sp->shp_type].m_maxair;

	} else if (isballoon(sp)) {

	    /*
	    ** Make balloons slowly lose gas.
	    ** Envelope loses (etus) units of gas per update.
	    ** Loss rate is twice as fast at high altitude.
	    ** At 75% of capacity, balloon cannot go to high altitude.
            ** At 50% of capacity, balloon cannot fly at all.
	    */

	    if (sp->shp_air < mchr[sp->shp_type].m_maxair/2) {
		sp->shp_depth = 0;
		if (sectp->sct_type < SCT_RURAL) {
		    wu(0, sp->shp_own,
		       fmt("%s crashed in %s\n",
			   shipas(sp,0),xyas(sp->shp_x,sp->shp_y,sp->shp_own)));
		    sp->shp_own = 0;
		    sp->shp_effic = 0;
		} else {
		    wu(0, sp->shp_own,
		       fmt("%s forced down in %s\n",
			   shipas(sp,0),xyas(sp->shp_x,sp->shp_y,sp->shp_own)));
		    if (sectp->sct_own && (sp->shp_own != sectp->sct_own)) {
			wu(0, sectp->sct_own,
			   fmt("%s captured after forced landing in %s\n",
			       shipas(sp,LO),xyas(sp->shp_x,sp->shp_y,
						  sectp->sct_own) ));
			sp->shp_own = sectp->sct_own;
		    }
		}
	    } else {
		if (sp->shp_air < mchr[sp->shp_type].m_maxair*3/4) {
		    if (sp->shp_depth > 1) {
			wu(0, sp->shp_own,
			   fmt("%s descending to lower altitude\n",
			       shipas(sp,0)));
			sp->shp_depth = 1;
		    }
		}
	    }

	    leakrate = 1.0 + 
		(double)mchr[sp->shp_type].m_maxair
		    * (double)(101 - sp->shp_effic) 
			* (double)etus
			    / (double)(sp->shp_tech+1) / 5.0;

	    if (sp->shp_depth > 1) leakrate = leakrate * 2.0;

	    sp->shp_air -= (int)(leakrate);
	    if (sp->shp_air < 0) sp->shp_air = 0;

	} else if ((n=vec[I_CIVIL]+vec[I_MILIT]+vec[I_UW])) { /* has crew */

	    /* 
	    **  Make submerged subs slowly use air.
	    **  Each soul aboard consumes (etus) units of air per update.
	    */

	    if (sp->shp_air == 0) {
		wu(0, sp->shp_own, 
		   fmt("Entire crew of %d suffocated on %s #%d\n",
		       n, mp->m_name, shipno));
		vec[I_CIVIL] = vec[I_MILIT] = vec[I_UW] = 0;
	    } else {
		if (n*etus > sp->shp_air) {
		    n -= (sp->shp_air/etus);
		    wu(0, sp->shp_own, fmt("%d suffocated on %s #%d\n",
					   n, mp->m_name, shipno));
		    while (n) {
			if (vec[I_UW]    && chance(0.6)) {n--;vec[I_UW]--;   }
			if (vec[I_CIVIL] && chance(0.5)) {n--;vec[I_CIVIL]--;}
			if (vec[I_MILIT] && chance(0.4)) {n--;vec[I_MILIT]--;}
		    }
		    n = sp->shp_air;
		} else {
		    n *= etus;
		}
		sp->shp_air -= n;
		wu(0,sp->shp_own,fmt("%d air used, %d remaining on %s #%d\n",
				     n,sp->shp_air,mp->m_name, shipno));
	    }
	}

	/*
	**  Do plague stuff.  plague can't break out on ships,
	**  but it can still kill people.
	*/
	getvec(VT_COND, cvec, (char *)sp, EF_SHIP);
	if (cvec[C_PSTAGE] > 0) {
		n = plague_people(np, vec, cvec, etus);
		switch (n) {
		case PLG_DYING:
			wu(0, sp->shp_own,
				fmt("PLAGUE deaths reported on %s #%d\n",
				mp->m_name, shipno));
			nreport(sp->shp_own, N_DIE_PLAGUE, 0, 1);
			break;
		case PLG_EXPOSED:
			wu(0, sp->shp_own, fmt("%s #%d battling PLAGUE\n",
				mp->m_name, shipno));
			break;
		case PLG_INCUBATE:
			if (n == cvec[C_PSTAGE])
				break;
			wu(0, sp->shp_own, fmt("Outbreak of PLAGUE on %s #%d!",
				mp->m_name, shipno));
			nreport(sp->shp_own, N_OUT_PLAGUE, 0, 1);
			break;
		default:
			break;
		}
		putvec(VT_COND, cvec, (char *)sp, EF_SHIP);
	}
	putvec(VT_ITEM, vec, (char *)sp, EF_SHIP);
	pops[sp->shp_own] += vec[I_CIVIL];
}


/* ________________________________________________________________
**|                                                                |
**| Idea is: a sector full of workers can fix up eight             |
**| battleships +8 % eff each etu.  This will cost around          |
**| 8 * 8 * $40 = $2560!                                           |
**|________________________________________________________________|
*/

int shiprepair(ship, vec, np, etus)
register struct shpstr *ship;
int *vec;
struct natstr *np;
int etus;
{
    extern double money_ship;
    register int delta;
    struct sctstr *sp;
    struct mchrstr *mp;
    int	wf;
    int	left;
    int	avail;
    int	w_p_eff;
    int	mult;
    
    mp = &mchr[ship->shp_type];
    sp = getsectp(ship->shp_x, ship->shp_y);

    /*
    **  It costs twice as much to repair things that are substantially
    **  beyond your current technical knowledge.
    */
    mult = (np->nat_level[NAT_TLEV] < ship->shp_tech * 0.85) ? 2 : 1;
    np->nat_money += (int) (mult * etus *
			    dmin(-1.0, money_ship * mp->m_cost));

    left = 100 - ship->shp_effic;
    if (left <= 0) return 1; /* ship is ok; no repairs needed */

    wf = 0;
    switch (mp->m_class) {
    case CLASS_TANK:
	/* Only military can work on armored & military vehicles. */
	if ((mp->m_armor>1) || (mp->m_glim>0)) wf = etus * vec[I_MILIT]/2;
	else wf = etus * (vec[I_CIVIL]/2 + vec[I_MILIT]/5);
	/* Tanks at sea and on foreign soil gain eff at 
	   1/3 normal rate; tanks otherwise not in owner's
	   tank farms gain at 1/2 normal rate. */
	if (!stypeinlist(class[mp->m_class].birthplace,sp->sct_type))
	    wf /= 2;
	if (sp->sct_own != ship->shp_own) 
	    wf = wf*5/6;
	break;
    case CLASS_SHIP:	
	/* Only military can work on a military boat */
	if (mp->m_glim > 0) wf = etus * vec[I_MILIT]/2;
	else wf = etus * (vec[I_CIVIL]/2 + vec[I_MILIT]/5);
	/* 
	**  Ships in foreign harbors do not gain eff (shore leave);
	**  ships at sea gain eff at only 1/3 the normal rate;
	**  submerged submarines gain eff at only 1/4 the normal rate.
	*/
	if (sp->sct_own && (sp->sct_own != ship->shp_own)) 
	    return 1;
	/* Original Code, new code fixed by jco@bbn.com Dec 91 */
	/* if (ship.shp_depth > 0) */ 
	if (ship->shp_depth > 0) 
	    wf /= 4;
	else if (!stypeinlist(class[mp->m_class].birthplace,sp->sct_type))
	    wf /= 3;
	break;
    default:
	/* Dedicated crews work on everything else */
	wf = etus * (vec[I_CIVIL]+vec[I_MILIT])/2;
	if (stypeinlist(class[mp->m_class].birthplace,sp->sct_type)
	    && (sp->sct_own == ship->shp_own))
	    break;
	return 1;
    }
    
    avail = wf + sp->sct_avail * 100;
    w_p_eff = 20 + (mp->m_lcm + 2 * mp->m_hcm);
    delta = roundavg((double)avail/w_p_eff);
    if (delta <= 0)   return 1;
    if (delta > etus) delta = etus;
    if (delta > left) delta = left;

    /* I didn't use roundavg here, because I want to penalize
       the player with a large number of ships. */

    wf -= delta * w_p_eff;
    if (wf < 0) {
	avail = (sp->sct_avail * 100 + wf) / 100;
	if (avail < 0) avail = 0;
	sp->sct_avail = avail;
    }
    np->nat_money -= mult * mp->m_cost * delta / 100.0;
    ship->shp_effic += delta;
    return 1;
}
